function [m_i, m_coef, m_residual]=ClassifyImages(dir_name, file_type, dict, W, param)
%This function select an image by random from the directory to test the first method 
%"Use the whole image for dictionary training".
%Author
%   Qiang Zhang
%   CIDSE, Arizona State University
%   qzhang53@asu.edu
%Description:
%   Just as the method in [1], but instead of manually choosing the dictionary, we use the
%   images to train the dictionary. First, we have Ni images for L people. Then we run the KSVD
%   for Ni images of every single people i, and obtain a dictionary of size Mi for i. All
%   these Mi sized dictionary consists the global dictionary D. Then for the test image, we
%   sparse code it, with method such as OMP, and get the coefficient A. We can identify the
%   test image buy find the i that minimizes ||y-deta(i, D*A)||.
%Operation:
%   next:      click left mouse for next image;
%   previoue:  click right mouse for previouse image;
%Parameter:
%   dir_name:  the directory of test image;
%   file_name: the name of test image;
%   dict:      the dictionary you want to use. The size of atom in dictionary should be the same with test image;
%   W:         classifier;
%   param:     some parameters used by the function;
%              no_people:   the number of people in the training sets;
%              support:     number of the non zero elements in the coefficient;
%              display:     display=1 to show the images;
%              save:        if save=1, the file will be stored.
%              sample:      the random sample matrix, or resize [x y];
%              number:      the number of random test, set to 0 if you want to test by hand. When number is -N, then
%                           it will run N test by random. When number is N, it will run N test in order You'd better
%                           set display to zero, when number is not zero.
%              test:        the file index that you want to test.
%   m_i:         the identify of the test image;
%   m_coeff:     the coefficient for the test image;
%   m_classify:  glossary result for classify.
%Caution:
%% ========================================================================
%go to the directory
cd(dir_name);
% get a dir list of all images of type 'imagetype'
eval( sprintf('dirlist=dir(''*.%s'');', file_type ) ); 
%total length
dict_len=length(dirlist);
i=1;
input=0;
if param.display
    figure;
end
%% ========================================================================
if isempty(param.test)==0
    m_i=zeros(1, size(param.test,2));
    m_coef=zeros(size(dict, 2), size(param.test,2));
    m_residual=zeros(param.no_people, size(param.test,2));
    c1=cputime
    for i=1:size(param.test,2)
        
        file_name=dirlist(param.test(i)).name;
        [ind, coef, residual]=ClassifyCoeff(file_name, dict, W, param);
%         [ind, coef, residual]=SrcKsvdOcc(file_name, dict, W, param);
        display(file_name);
        display(ind);
        m_i(i)=ind;
        m_coef(:,i)=coef;
        m_residual(:,i)=residual';
    end
    c2=cputime-c1
else
    if param.number==0
        while input~='q'
            if input==1
                if i==dict_len
                    i=1;
                else
                    i=i+1;
                end
                file_name=dirlist(i).name;
%                 [ind, coef, residual]=SrcKsvdOcc(file_name, dict, W, param);
                [ind, coef, residual]=ClassifyCoeff(file_name, dict, W, param);
                display(file_name);
                display(ind);
            else if input==3
                    if i==1
                        i=dict_len;
                    else
                        i=i-1;
                    end
                    file_name=dirlist(i).name;
                    [ind, coef, residual]=ClassifyCoeff(file_name, dict, W, param);
%                     [ind, coef, residual]=SrcKsvdOcc(file_name, dict, W, param);
                    display(file_name);
                    display(ind);
                else if input=='r'
                    i=ceil(rand(1, 1)*dict_len);
                    file_name=dirlist(i).name;
%                     [ind, coef, residual]=SrcKsvdOcc(file_name, dict, W, param);
                    [ind, coef, residual]=ClassifyCoeff(file_name, dict, W, param);
                    display(file_name);
                    display(ind);
                    end
                end
            end
            [x,y,input]=ginput(1);
            m_i=[m_i ind];
            m_coef=[m_coef coef];
            m_residual=[m_residual residual'];
        end
        else
            m_i=zeros(1, abs(param.number));
            m_coef=zeros(size(dict, 2), abs(param.number));
            m_residual=zeros(param.no_people, abs(param.number));

            if param.number<0
    c1=cputime
                for j=param.number:-1;
                    i=ceil(rand(1, 1)*dict_len);
                    file_name=dirlist(i).name;
%                     [ind, coef, residual]=SrcKsvdOcc(file_name, dict, W, param);
                    [ind, coef, residual]=ClassifyCoeff(file_name, dict, W, param);
                    display(file_name);
                    display(ind);
                    m_i(:,-j)=ind;
                    m_coef(:,-j)=coef;
                    m_residual(:,-j)=residual';
                    end
    c2=cputime-c1
            else
    c1=cputime
                for i=1:param.number;
                    file_name=dirlist(i).name;
                    [ind, coef, residual]=ClassifyCoeff(file_name, dict, W, param);
%                     [ind, coef, residual]=SrcKsvdOcc(file_name, dict, W, param);
                    display(file_name);
                    display(ind);
                    m_i(:,i)=ind;
                    m_coef(:,i)=coef;
                    m_residual(:,i)=residual';
                end
    c2=cputime-c1
            end
    end
end
