function [i, coeff, residual]=SRCTestCoeff(file_name, dict, param)
%This function recognize the face image with the first method "Use the whole image for dictionary training".
%Author
%   Qiang Zhang
%   CIDSE, Arizona State University
%   qzhang53@asu.edu
%Description:
%   Just as the method in [1], but instead of manually choosing the dictionary, we use the
%   images to train the dictionary. First, we have Ni images for L people. Then we run the KSVD
%   for Ni images of every single people i, and obtain a dictionary of size Mi for i. All
%   these Mi sized dictionary consists the global dictionary D. Then for the test image, we
%   sparse code it, with method such as OMP, and get the coefficient A. We can identify the
%   test image buy find the i that minimizes ||y-deta(i, D*A)||.
%Parameter:
%   file_name: the name of test image;
%   dict:      the dictionary you want to use. The size of atom in dictionary should be the same with test image;
%   param:     some parameters used by the function;
%              no_people:   number of people in the dictionary;
%              no_atom:     number of atoms per people in the dictionary.
%              support:     number of the non zero elements in the coefficient;
%              display:     display=1 to show the images;
%              save:        if save=1, the file will be stored.
%              sample:      random sample matrix
%   i:         the identify of the test image;
%   coeff:     the coefficient for the test image;
%Caution:

%read in the image
img=imread(file_name);
if size(img, 3)>1;
    img=rgb2gray(img);
end
img=double(img);
[row, col]=size(img);
img_len=row*col;

%use the omp for sparse coding
if size(param.sample, 1)>1
    img=reshape(img, img_len, 1);
    img=param.sample*img;
else
    img=imresize(img, param.sample);
    [row, col]=size(img);
    img_len=row*col;
    img=reshape(img, img_len, 1);
end
coeff=OMP(dict, img, param.support);

%identify the test image
%construct a matrix that select coefficient just like function deta
m_coeff=zeros(param.no_atom * param.no_people, param.no_people);
for i=1:param.no_people
    m_coeff((i-1)*param.no_atom+1 : i*param.no_atom, i)=coeff((i-1)*param.no_atom+1 : i*param.no_atom);
end
m_img=img*ones(1, param.no_people);
residual=zeros(1, param.no_people);
tmp=m_img-dict*m_coeff;
for i=1:param.no_people
    residual(i)=norm(tmp(:,i),2);
end
[c, i]=min(residual);
c=c/norm(img);

if param.display==1
    colormap(gray);
    bar(full(coeff));
    title('coefficient');
    if param.save==1
        saveas(gcf, file_name(1:20), 'jpg');
    end
end
disp(sprintf('reconstruction error normalized by orignal image is: %f .\n', c));