function [dict, output]=SRCTrain(dir_name, file_type, param)
%src
%Author
%   Qiang Zhang
%   CIDSE, Arizona State University
%   qzhang53@asu.edu
%Description:
%   Just as the method in [1], but instead of manually choosing the dictionary, we use the
%   images to train the dictionary. First, we have Ni images for L people. Then we run the KSVD
%   for Ni images of every single people i, and obtain a dictionary of size Mi for i. All
%   these Mi sized dictionary consists the global dictionary D. Then for the test image, we
%   sparse code it, with method such as OMP, and get the coefficient A. We can identify the
%   test image buy find the i that minimizes ||y-deta(i, D*A)||.
%Parameter:
%   dir_name:  the name of directory where you store the training set;
%   file_type: type of file you want to use as training set;
%   param:
%              no_people: number of the people in the training set;
%              no_atom:   number of atoms in the trained dictionary for every people;
%              ind_image: the index of image that you want to use as the atom.
%              support:   the non zero elements in the coeffcient;
%              sample:    the random sample matrix;
%   dict:      output the trained dictionary. The atom is organized as columns every atom is a trained picture.
%Caution: 
%   Please set your directory first, because this function willprocess all the image in the directory.
%   All the images should be 8 bit gray and take the same size

%go to the directory
cd(dir_name);
% get a dir list of all images of type 'imagetype'
eval( sprintf('dirlist=dir(''*.%s'');', file_type ) ); 

output=[];
%parameter for ksvd
%K is the number of atoms in dictionary
%K should be much smaller than the size of a image. It is independent from
%the number of images for each people.
%L is the non zero element in coefficient. L should be (much) smaller than
%K.
if isempty(param.ind_image)==0
    if size(param.sample, 1)>1
        %has to do some random sample
        img=imread(dirlist(param.ind_image(1)).name);
        len_img=size(img, 1)*size(img, 2);
        dict=zeros(size(param.sample, 1), length(dirlist));
        for i=1:length(dirlist)
            img=imread(dirlist(param.ind_image(i)).name);
            if size(img,3)>1
                img=rgb2gray(img);
            end
            tmp=param.sample*double(reshape(img, len_img, 1));
            dict(:, i)=tmp/norm(tmp);
        end
    else
        %first get the size of images
        len_img=param.sample(1)*param.sample(2);
        dict=zeros(param.sample(1)*param.sample(2), length(dirlist));
        for i=1:length(dirlist)
            img=imread(dirlist(param.ind_image(i)).name);
            if size(img,3)>1
                img=rgb2gray(img);
            end
            img=imresize(img, param.sample);
            tmp=double(reshape(img, len_img, 1));
            dict(:, i)=tmp/norm(tmp);
        end
    end
else
    if size(param.sample, 1)>1
        %has to do some random sample
        img=imread(dirlist(1).name);
        len_img=size(img, 1)*size(img, 2);
        dict=zeros(size(param.sample, 1), length(dirlist));
        for i=1:length(dirlist)
            img=imread(dirlist(i).name);
            if size(img,3)>1
                img=rgb2gray(img);
            end
            tmp=param.sample*double(reshape(img, len_img, 1));
            dict(:, i)=tmp/norm(tmp);
        end
    else
        %first get the size of images
        len_img=param.sample(1)*param.sample(2);
        dict=zeros(param.sample(1)*param.sample(2), length(dirlist));
        for i=1:length(dirlist)
            img=imread(dirlist(i).name);
            if size(img,3)>1
                img=rgb2gray(img);
            end
            img=imresize(img, param.sample);
            tmp=double(reshape(img, len_img, 1));
            dict(:, i)=tmp/norm(tmp);
        end
    end
end

